@@ -56,14 +56,22 @@ brl_ser_open_port (gint port)
 
     switch (port)
     {
-	case 1:
+	case 0:
 	    pname = TTYNAME0; 	
 	break;
     
-	case 2:
+	case 1:
 	    pname = TTYNAME1;
 	break;
 	
+	case 2:
+	    pname = TTYNAME2; 	
+	break;
+    
+	case 3:
+	    pname = TTYNAME3;
+	break;
+
 	default:
 	    fprintf (stderr, "\nbrl_open_port: Invalid serial port number %d", port);
 	    return 0;
