@@ -308,7 +308,7 @@ srs_gs_get_driver_voices (gchar *driver)
 
 
 gboolean
-srs_gs_init (SRSGSMarkersCallback callback)
+srs_gs_init (SRSGSMarkersCallback callback, SRSResetCallback reset_callback)
 {
     sru_assert (callback);
 
@@ -316,7 +316,7 @@ srs_gs_init (SRSGSMarkersCallback callback)
     srs_gs_outs = NULL;
     srs_gs_start_marker_waiting = FALSE;
     
-    return srs_gs_wrap_init (srs_gs_callback);
+    return srs_gs_wrap_init (srs_gs_callback, reset_callback);
 }
 
 void
