@@ -38,7 +38,13 @@
 #include "srintl.h"
 #include "screen-review.h"
 #include <libxml/parser.h>
+#include "sru-debug.h"
 
+/* mouse motion idle support */
+#define SRC_MOUSE_MOVED_TIMEOUT		300 /* ms */
+static gboolean src_mouse_is_moving = FALSE;
+static guint mouse_stopped_moving_timeout_handler = 0;
+static guint mouse_is_moving_idle_handler = 0;
 
 gboolean login_time;
 
@@ -55,7 +61,7 @@ gboolean src_enable_format_string = FALSE;
 gint src_caret_position;
 
 gchar *braille_device = NULL;
-gint   braille_port;
+gint   braille_port = -1;
 
 extern gchar *scroll;
 extern gint old_offset;
@@ -96,6 +102,9 @@ src_present_real_sro_for_brl_and_brlmon (SRObject *obj, gchar *reason)
 
     srl_assert (src_use_braille || src_use_braille_monitor);
 
+    if ((src_use_speech || src_use_magnifier) && srl_is_object_watched (obj))
+	return;
+
 	src_caret_position = -1;
     src_enable_format_string = FALSE; 
     txt = src_presentation_sro_get_pres_for_device (obj, reason, "braille");
@@ -258,8 +267,7 @@ src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
 	{
 	    SRObjectRoles role;
 	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	    if (srl_is_object_focused (obj) || strcmp (reason, "present") == 0 ||
-		strcmp (reason, "object:context-switched") == 0)
+	    if (srl_is_object_focused (obj) || strcmp (reason, "present") == 0)
 	    {
 		if (role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
 		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_WARNING, TRUE);
@@ -268,10 +276,10 @@ src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
 		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, FALSE);
 		else
 		{
-		    gboolean interrupt = TRUE;
 		    if (strcmp (reason, "object:text-changed:insert") == 0 && role == SR_ROLE_TERMINAL)
-			interrupt = FALSE;
-		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, interrupt);
+			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_LOW, FALSE);
+		    else
+			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 		}
 	    }
 	    else if (srl_is_object_watched (obj))
@@ -290,6 +298,12 @@ src_present_real_sro_for_magnifier (SRObject *obj, gchar *reason)
 
     srl_assert (src_use_magnifier);
 
+    if (src_use_speech && srl_is_object_watched (obj))
+	return;
+
+    if (src_mouse_is_moving)
+	return;
+
     txt = src_presentation_sro_get_pres_for_device (obj, reason, "magnifier");
     if (txt)
     {
@@ -387,6 +401,8 @@ src_present_crt_window ()
 		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_WARNING, FALSE);
 	    else if (strcmp (reason, "window:rename") == 0)
 		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_LOW, TRUE);
+	    else if (strcmp (reason, "object:context-switched") == 0)
+	    	src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_WARNING, TRUE);
 	    else
 		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
 	}
@@ -1686,6 +1702,16 @@ src_process_modifier (gchar *modifier)
 	    part1 = _("shift");
 	    pos = 6;
         }
+	else if (strncmp (modifier, "Super", 5) == 0)
+	{
+	    part1 = _("Window");
+	    pos = 6;
+        }	
+	else if (strncmp (modifier, "Hyper", 5) == 0)
+	{
+	    part1 = _("Menu");
+	    pos = 6;
+        }	
 	else if (strncmp (modifier, "Caps", 3) == 0)
 	{
 	    part1 = _("caps lock");
@@ -1757,6 +1783,8 @@ src_process_cursor (gchar *cursor)
 	    src_last_cursor_echo = g_strdup (_("delete"));
 	else if (strcmp (cursor, "Escape") == 0)
 	    src_last_cursor_echo = g_strdup (_("escape"));	    
+	else if (strcmp (cursor, "Menu") == 0)
+	    src_last_cursor_echo = g_strdup (_("Menu"));
 	else
 	    sru_assert_not_reached ();
 	        
@@ -1798,7 +1826,9 @@ src_process_key (SREvent *event,
 	strncmp (srhotkey_data->keystring, "Control", 7) == 0 ||
 	strncmp (srhotkey_data->keystring, "Shift", 5) == 0 ||
 	strncmp (srhotkey_data->keystring, "Caps", 3) == 0 ||
-	strncmp (srhotkey_data->keystring, "Num", 3) == 0)
+	strncmp (srhotkey_data->keystring, "Num", 3) == 0 ||
+	strncmp (srhotkey_data->keystring, "Super", 5) == 0 ||
+	strncmp (srhotkey_data->keystring, "Hyper", 5) == 0)
     {
     
 	src_process_modifier (srhotkey_data->keystring);
@@ -1813,7 +1843,8 @@ src_process_key (SREvent *event,
 	     strcmp (srhotkey_data->keystring, "Up") == 0 ||
 	     strcmp (srhotkey_data->keystring, "BackSpace") == 0 ||
 	     strcmp (srhotkey_data->keystring, "Delete") == 0 ||
-	     strcmp (srhotkey_data->keystring, "Escape") == 0)
+	     strcmp (srhotkey_data->keystring, "Escape") == 0 ||
+	     strcmp (srhotkey_data->keystring, "Menu") == 0)
     {
 	src_process_cursor (srhotkey_data->keystring);
     }
@@ -1963,6 +1994,27 @@ src_process_tooltip_event (SREvent *event,
     }
 }
 
+static gboolean
+src_mouse_moved_timeout (gpointer data)
+{
+    src_mouse_is_moving = FALSE;
+    return FALSE;
+}
+
+static gboolean
+src_mouse_moved_set_timeout (gpointer data)
+{
+    if (mouse_stopped_moving_timeout_handler) 
+    {
+	g_source_remove (mouse_stopped_moving_timeout_handler);
+    }
+    mouse_stopped_moving_timeout_handler = 
+	g_timeout_add (SRC_MOUSE_MOVED_TIMEOUT, src_mouse_moved_timeout, NULL);
+
+    mouse_is_moving_idle_handler = 0;
+    return FALSE;
+}
+
 static void
 src_process_mouse_event (SREvent *event,
 		 unsigned long flags)
@@ -1971,7 +2023,12 @@ src_process_mouse_event (SREvent *event,
 
     if (!src_use_magnifier)
 	return;
-	
+
+    src_mouse_is_moving = TRUE;
+    if (!mouse_is_moving_idle_handler)
+    {
+	mouse_is_moving_idle_handler = g_idle_add (src_mouse_moved_set_timeout, NULL);
+    }
     if (sre_get_event_data (event, (void**)&point))
     {
 	gchar *magout;
@@ -2266,6 +2323,20 @@ src_set_command_lines (const gchar *mouse_tracking,
     }
 }    
 
+static gboolean
+src_login_focus (gpointer data)
+{
+    SRObject *focus;
+
+    focus = srl_get_focus_object ();
+    if (focus)
+    {
+	src_process_sro (focus);
+	sro_release_reference (focus);
+    }
+    return FALSE;
+}
+
 static void
 src_init(const gchar *mouse_tracking,
 	 const gchar *focus_tracking,
@@ -2280,6 +2351,8 @@ src_init(const gchar *mouse_tracking,
     gboolean initialize_finished = FALSE;
     gboolean default_invert_value;
 
+    sru_debug_init ();
+
     /* initialize SR Utils */
     sru_init();
 
@@ -2295,7 +2368,7 @@ src_init(const gchar *mouse_tracking,
     if (braille_device)
 	src_braille_set_device (braille_device);
     
-    if (braille_port)
+    if (braille_port != -1)
 	src_braille_set_port_no (braille_port);
     /* initialize 
     
@@ -2360,6 +2433,7 @@ src_init(const gchar *mouse_tracking,
     src_crt_sro = NULL;
     src_crt_tooltip = NULL;
     src_crt_window = NULL;
+    src_mouse_is_moving = FALSE;
 
     src_message = NULL;
 
@@ -2385,6 +2459,8 @@ src_init(const gchar *mouse_tracking,
     SET_SRCORE_CONFIG_DATA(SRCORE_INITIALIZATION_END, CFGT_BOOL, &initialize_finished);
     initialize_finished = !initialize_finished;
     SET_SRCORE_CONFIG_DATA(SRCORE_INITIALIZATION_END, CFGT_BOOL, &initialize_finished);    
+    if (login_time)
+	g_idle_add (src_login_focus, NULL);
 }
 
 static void
@@ -2463,6 +2539,8 @@ src_terminate()
 	g_free (src_last_modifier_echo);
     if (src_last_cursor_echo)
 	g_free (src_last_cursor_echo);
+
+    sru_debug_terminate ();
 }
     
 gint
@@ -2686,8 +2764,8 @@ main (gint argc,
     src_use_magnifier = (disable_magnifier ? FALSE :
 			(enable_magnifier  ? TRUE : -1));
 
-    if (braille_port <= MIN_BRAILLE_PORT && 
-	braille_port >= MAX_BRAILLE_PORT)
+    if (braille_port < MIN_SERIAL_PORT_NO ||
+	braille_port > MAX_SERIAL_PORT_NO)
     {
 	braille_port = DEFAULT_BRAILLE_PORT_NO;
     }
